#include     "UGAlib.mqh"
#include     "MustHave.mqh"
#include     "StrategyMA.mqh"                                        //Strategy 0 - moving averages
#include     "StrategySAR.mqh"                                       //Strategy 1 - SAR Breakthrough
#include     "StrategyStoch.mqh"                                     //Strategy 2 - stochastic

double       cap=10000.0;                                            //Initial deposit
long         leverage;                                               //Account leverage
double       contractSize;                                           //Contract size
int          z;                                                      //Intermediary between a gene colony and optimied parameters

double       ERROR=0.0;                                              //Average error per gene (that is for the genetic optimizer,,...
                                                                     //...I don't know the value)

void FitnessFunction(int chromos)                                    //Fitness function for the genetic optimizer:...
                                                                     //...selects a strategy, symbol, deposit size,...
                                                                     //...parameters of indicator buffers;...
                                                                     //...you can optimize whatever you need, but...
                                                                     //...watch carefully the number if genes
{
  double ff=0.0;                                                     //Fitness function proper
  strat=(int)MathRound(Colony[GeneCount-2][chromos]*StratCount);     //GA chooses a strategy
//For the EA testing mode, use the following code...
  z=(int)MathRound(Colony[GeneCount-1][chromos]*3);                  //GA selects a symbol
  switch(z)
  {
    case  0: {s="EURUSD"; break;};
    case  1: {s="GBPUSD"; break;};
    case  2: {s="USDCHF"; break;};
    case  3: {s="USDJPY"; break;};
    default: {s="EURUSD"; break;};
  }
//..For a real mode, comment the previous code and uncomment the below one (symbols are selected in the MarketWatch window)
/*
  z=(int)MathRound(Colony[GeneCount-1][chromos]*(SymbolsTotal(true)-1));//GA selects a symbol
  s=SymbolName(z,true);
*/
  optF=Colony[GeneCount][chromos];                                   //GA selects deposit share
  switch(strat)
  {
    case  0: {ff=FFMA(   Colony[1][chromos],                         //The number of case strings must be equal to the number of strategies
                         Colony[2][chromos],
                         Colony[3][chromos],
                         Colony[4][chromos],
                         Colony[5][chromos]); break;};
    case  1: {ff=FFSAR(  Colony[1][chromos],
                         Colony[2][chromos],
                         Colony[3][chromos],
                         Colony[4][chromos],
                         Colony[5][chromos]); break;};
    case  2: {ff=FFStoch(Colony[1][chromos],
                         Colony[2][chromos],
                         Colony[3][chromos],
                         Colony[4][chromos],
                         Colony[5][chromos]); break;};
    default: {ff=FFMA(   Colony[1][chromos],
                         Colony[2][chromos],
                         Colony[3][chromos],
                         Colony[4][chromos],
                         Colony[5][chromos]); break;};
  }
  AmountStartsFF++;
  Colony[0][chromos]=ff;
  Print(TimeToString(TimeCurrent()),";","GAModule:FitnessFunction",
        ";","strat=",strat,";","s=",s,";","optF=",optF,
        ";",Colony[1][chromos],";",Colony[2][chromos],";",Colony[3][chromos],";",Colony[4][chromos],";",Colony[5][chromos]);
}

void ServiceFunction()
{ 
}

void GA()                                                            //Preparing and calling the genetic optimizer
{
  GeneCount      =OptParamCount+3;                                   //Number of genes (equal to the number of optimized indicator buffers,...
                                                                     //...plus 3 to chose a strategy, symbol, deposit share;...
                                                                     //... all of them should be specified in FitnessFunction())
  ChromosomeCount=GeneCount*11;                                      //Number of chromosomes in colony
  RangeMinimum   =0.0;                                               //Minimum of the search range
  RangeMaximum   =1.0;                                               //Maximum of the search range
  Precision      =0.0001;                                            //Search step
  OptimizeMethod =2;                                                 //1-minimum, any other-maximum
  ArrayResize    (Chromosome,GeneCount+1);
  ArrayInitialize(Chromosome,0);
  Epoch          =100;                                               //Number of epochs without improvement
  UGA(100.0,1.0,1.0,1.0,1.0,0.5,1.0);                                //Ratio of replication, natural mutation, artificial mutation, gene borrowing, crossingover, interval boundary displacement ratio, every gene mutation probability, %
}

void GetTrainResults()                                               //Get the best parameters
{
  strat=(int)MathRound(Chromosome[GeneCount-2]*StratCount);          //Remember the best strategy
//For the EA testing mode, use the following code...
  z=(int)MathRound(Chromosome[GeneCount-1]*3);                       //Remember the best symbol
  switch(z)
  {
    case  0: {s="EURUSD"; break;};
    case  1: {s="GBPUSD"; break;};
    case  2: {s="USDCHF"; break;};
    case  3: {s="USDJPY"; break;};
    default: {s="EURUSD"; break;};
  }
//..For a real mode, comment the previous code and uncomment the below one (symbols are selected in the MarketWatch window)
/*
  z=(int)MathRound(Chromosome[GeneCount-1]*(SymbolsTotal(true)-1));  //Remember the best symbol
  s=SymbolName(z,true);
*/
  optF=Chromosome[GeneCount];                                        //Remember the best deposit share
  switch(strat)
  {
    case  0: {GTRMA(   Chromosome[1],                                //The number of case strings must be equal to the number of strategies
                       Chromosome[2],
                       Chromosome[3],
                       Chromosome[4],
                       Chromosome[5]) ; break;};
    case  1: {GTRSAR(  Chromosome[1],
                       Chromosome[2],
                       Chromosome[3],
                       Chromosome[4],
                       Chromosome[5]) ; break;};
    case  2: {GTRStoch(Chromosome[1],
                       Chromosome[2],
                       Chromosome[3],
                       Chromosome[4],
                       Chromosome[5]) ; break;};
    default: {GTRMA(   Chromosome[1],
                       Chromosome[2],
                       Chromosome[3],
                       Chromosome[4],
                       Chromosome[5]) ; break;};
  }
  Print(TimeToString(TimeCurrent()),";","GAModule:GetTrainResults",
        ";","strat=",strat,";","s=",s,";","optF=",optF,
        ";",Chromosome[1],";",Chromosome[2],";",Chromosome[3],";",Chromosome[4],";",Chromosome[5]);
}